# -*- coding: utf-8 -*-


from base import BaseOAuth2


class Renren(BaseOAuth2):

    AUTHORIZATION_ENDPOINT = 'https://graph.renren.com/oauth/authorize'
    TOKEN_ENDPOINT = 'https://graph.renren.com/oauth/token'
    API_URL = 'https://api.renren.com/v2'

    ERROR_CODE_PARAM_NAME = 'error'
    ERROR_MSG_PARAM_NAME = 'error_description'

    def get_user_info(self, result, **kwargs):
        self.uid = result['user']['id']
        self.access_token = result['access_token']
        self.refresh_token = result['refresh_token']
        self.expires_in = result['expires_in']

        self.client.set_access_token(self.access_token)
        result = self.fetch('/user/get', kwargs)
        self.name = result[0]['name']
        self.avatar = result[0]['tinyurl']
        self.avatar_large = result[0]['headurl']



# -*- coding: utf-8 -*-


from base import BaseOAuth2


class TencentWeibo(BaseOAuth2):

    AUTHORIZATION_ENDPOINT = 'https://open.t.qq.com/cgi-bin/oauth2/authorize'
    TOKEN_ENDPOINT = 'https://open.t.qq.com/cgi-bin/oauth2/access_token'
    API_URL = 'http://open.t.qq.com/api'

    ERROR_CODE_PARAM_NAME = 'errorCode'
    ERROR_MSG_PARAM_NAME = 'errorMsg'

    def _error_handler(self, resp, error_class, url):
        '''
        腾讯微博比较蛋疼
        认证和api接口的错误键不一样
        '''
        if self.ERROR_MSG_PARAM_NAME in resp.result:
            errno = resp.result[self.ERROR_CODE_PARAM_NAME]
            error = resp.result[self.ERROR_MSG_PARAM_NAME]
            raise error_class('request url: %s, error code: %s, error message: %s' % (url, errno, error))
        elif 'errcode' in resp.result:
            errno = resp.result['errcode']
            if errno:
                error = resp.result['msg']
                raise error_class('request url: %s, error code: %s, error message: %s' % (url, errno, error))

    def get_user_info(self, result, **kwargs):
        self.uid = result['openid']
        self.access_token = result['access_token']
        self.refresh_token = result['refresh_token']
        self.expires_in = result['expires_in']
        self.name = result['nick']

        self.client.set_access_token(self.access_token)
        params = dict(oauth_consumer_key=self.client_id, oauth_version='2.a', openid=self.uid, format='json')
        params.update(kwargs)
        result = self.fetch('/user/info', params)
        self.avatar = result['data']['head']
        self.avatar_large = ""



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from setuptools import setup, find_packages


setup(
        name = 'pysns',
        version = '0.0.2',
        keywords = ('OAuth2 Client', 'OAuth2', 'Login'),
        description = 'Third-party login',
        long_description = 'See http://github.com/zakzou/pysns',
        license = 'MIT',

        url = 'http://github.com/zakzou/pysns',
        author = 'zakzou',
        author_email = 'zakzou@live.com',

        packages = find_packages(),
        include_package_data = True,
        platforms = 'any',
        install_requires = ['simplejson>=2.3.2', 'liboauth2>=0.0.1'],
        classifiers=[
            "Environment :: Web Environment",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
            "Programming Language :: Python",
            ],
        )



# -*- coding: utf-8 -*-


import os
import sys


__version__ = '0.0.2'


ROOT_DIR = os.path.dirname(os.path.realpath(__file__))
sys.path.insert(0, ROOT_DIR)


def get_client(name, *args, **kwargs):
    '''
    获取认证客户端

    client = get_client('weibo', client_id, client_secret, redirect_uri)
    '''
    return get_client_class(name)(*args, **kwargs)


def get_client_class(name, **kwargs):
    '''
    获取认证客户端类

    cls = get_client_class('tencent_weibo')
    client = cls(client_id, client_secret, redirect_uri)
    '''
    cls_name = _get_cls_name(name)
    module = __import__('adapter.%s' % name, fromlist=[cls_name])
    return getattr(module, cls_name)


def _get_cls_name(name):
    items = []
    for item in name.split('_'):
        items.append(item[:1].upper() + item[1:])
    return ''.join(items)



# -*- coding: utf-8 -*-


from base import BaseOAuth2


class Weibo(BaseOAuth2):

    AUTHORIZATION_ENDPOINT = 'https://api.weibo.com/oauth2/authorize'
    TOKEN_ENDPOINT = 'https://api.weibo.com/oauth2/access_token'
    API_URL = 'https://api.weibo.com/2'

    ERROR_CODE_PARAM_NAME = 'error_code'
    ERROR_MSG_PARAM_NAME = 'error'

    def get_user_info(self, result, **kwargs):
        self.uid = result['uid']
        self.access_token = result['access_token']
        self.refresh_token = ''
        self.expires_in = result['expires_in']

        self.client.set_access_token(self.access_token)
        params = dict(uid=self.uid)
        params.update(kwargs)
        result = self.fetch('/users/show.json', params)
        self.name = result['name']
        self.avatar = result['profile_image_url']
        self.avatar_large = result['avatar_large']



# -*- coding: utf-8 -*-


import liboauth2

from base import BaseOAuth2


class Douban(BaseOAuth2):

    AUTHORIZATION_ENDPOINT = 'https://www.douban.com/service/auth2/auth'
    TOKEN_ENDPOINT = 'https://www.douban.com/service/auth2/token'
    API_URL = 'https://api.douban.com/v2'

    ERROR_CODE_PARAM_NAME = 'code'
    ERROR_MSG_PARAM_NAME = 'msg'

    def get_user_info(self, result, **kwargs):
        self.uid = result['douban_user_id']
        self.access_token = result['access_token']
        self.refresh_token = result['refresh_token']
        self.expires_in = result['expires_in']

        self.client.set_access_token(self.access_token)
        self.client.set_access_token_type(liboauth2.ACCESS_TOKEN_BEARER)
        result = self.fetch('/user/~me', kwargs)
        self.name = result['name']
        self.avatar = result['avatar']
        self.avatar_large = ""



# -*- coding: utf-8 -*-


import liboauth2
try:
    import simplejson as json
except ImportError:
    import json

from urllib import unquote
from exception import OAuth2Error, ApiError


class Response(object):

    def __init__(self, code, result, content_type):
        self.code = code
        self.result = result
        self.content_type = content_type


class BaseOAuth2(object):
    '''
    认证包装基础类

    大部分OAuth2发生错误时候，status code都是400
    目前发现腾讯微博发生错误的时候 status code也是200
    所以需要重写方法_error_handler
    '''

    AUTHORIZATION_ENDPOINT = ''
    TOKEN_ENDPOINT = ''
    API_URL = ''

    ERROR_CODE_PARAM_NAME = ''
    ERROR_MSG_PARAM_NAME = ''


    def __init__(self, client_id, client_secret, redirect_uri):
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = redirect_uri
        self.client = liboauth2.Client(self.client_id, self.client_secret)

    def get_auth_url(self, **kwargs):
        return self.client.get_auth_url(self.AUTHORIZATION_ENDPOINT, self.redirect_uri, **kwargs)

    def get_access_token(self, code, **kwargs):
        params = dict(code=code, redirect_uri=self.redirect_uri)
        params.update(kwargs)
        resp = self.client.get_access_token(self.TOKEN_ENDPOINT, liboauth2.GRANT_TYPE_AUTH_CODE, params)
        resp = self._proc_resp(resp)
        self._error_handler(resp, OAuth2Error, self.TOKEN_ENDPOINT)
        return resp.result

    def get_user_info(self, result, **kwargs):
        raise NotImplementedError

    def fetch(self, url, params={}, http_method=liboauth2.HTTP_METHOD_GET, http_headers={}):
        url = self.API_URL + url
        resp = self.client.fetch(url, params, http_method, http_headers, liboauth2.HTTP_FORM_CONTENT_TYPE_APPLICATION)
        resp = self._proc_resp(resp)
        self._error_handler(resp, ApiError, url)
        return resp.result

    def _error_handler(self, resp, error_class, url):
        if resp.code != 200:
            errno = resp.result[self.ERROR_CODE_PARAM_NAME]
            error = resp.result[self.ERROR_MSG_PARAM_NAME]
            raise error_class('request url: %s, error code: %s, error message: %s' % (url, errno, error))

    def urldecode(self, values):
        ret = {}
        for one in values.split('&'):
            key, value = map(unquote, one.split('='))
            ret[key] = value
        return ret

    def jsondecode(self, values):
        return json.loads(values)

    def _proc_resp(self, resp):
        if resp['content_type'] and 'urlencoded' in resp['content_type']:
            resp['result'] = self.urldecode(resp['result'])
        else:
            resp['result'] = self.jsondecode(resp['result'])
        return Response(**resp)
